-- DESCRIBING SECTION =============================================================================
-------------------------------------------------------------------------------- 
-- 1. SOFTWAROVA s.r.o.
-- (c) 2012
--------------------------------------------------------------------------------
-- eXtreme PARALLEL PROCESSING
--------------------------------------------------------------------------------
-- Requirements:
-- Oracle Database Enterprise version >= 11g
-- CPU >= 1, Mem, Cluster
-- Sufficient size of REDO LOG is required.
--------------------------------------------------------------------------------
-- Run as user defined in XPP_OWNER
-- =============================================================================
------------- S C R I P T    N A M E -----------
DEF patch_type   = du
DEF patch_number = 00000002
DEF step_order   = 03
DEF rest_of_name = _install_xpp
------------- S C R I P T    I N F O -----------
DEF description     = 'Installation script XPP'
DEF version         = 1.0
DEF created_by      = '1SW'
DEF create_date     = 07.06.2012
DEF affected_apps   = 'XPP'
REM for             = SQL*PLUS
REM parameters      = none

----------- S C R I P T    H I S T O R Y --------
REM     Version   When             Who               What
REM     1.0       2012/05/25       Petr Jezek        Creation
-- INITIAL UNIVERSAL SECTION ====================================================

@../du00000002_setting_env_xpp.sql

DEF script_name  = &patch_type&patch_number._&step_order&rest_of_name
Set SERVEROUTPUT On Size 1000000
SPOOL &script_name..log
Set VERIFY Off
Set TRIMSPOOL On
PROMPT
PROMPT *************************************************************************
PROMPT SCRIPT FILE: &script_name..sql
PROMPT LOG    FILE: &script_name..log
PROMPT SCRIPT DESC: &description
PROMPT SCRIPT VER.: &version
PROMPT *************************************************************************
PROMPT
PROMPT Affected Application(s) : &affected_apps
PROMPT Executing ...
PROMPT
-- EXECUTIVE SECTION ==============================================================================
Set FEEDBACK Off
Declare
  v_db_user VARCHAR2(500);
  v_net_user VARCHAR2(500);
Begin 
   -- Common actions
   SELECT 'Database user: ' || user || '@' || sys_context('USERENV', 'DB_NAME')
     INTO v_db_user 
     FROM dual;
   SELECT 'Network user: ' || sys_context('USERENV', 'OS_USER') || '@' || sys_context('USERENV', 'HOST')
     INTO v_net_user 
     FROM dual;
   DBMS_OUTPUT.put_line ('------------------------------------------------------------');
   DBMS_OUTPUT.put_line ('Start ' || TO_CHAR(SYSDATE,'DD.MM.YYYY HH24:MI:SS') ) ;
   DBMS_OUTPUT.put_line(v_db_user);
   DBMS_OUTPUT.put_line(v_net_user);   
   DBMS_OUTPUT.put_line ('------------------------------------------------------------');
   DBMS_OUTPUT.put_line ('	') ;
End; 
/
Set FEEDBACK On
Set DEF on
SET SERVEROUTPUT ON SIZE UNLIMITED

prompt create table xpp_patches
create table xpp_patches
(
   seq_number    number(6)
  ,patch_type    varchar2(2 byte)
  ,patch_number  number(5)
  ,script_name   varchar2(200 byte)
  ,created_by    varchar2(50 byte)
  ,create_date   date
  ,applied_by    varchar2(50 byte)
  ,apply_date    date
  ,description   varchar2(2000 byte)
)
tablespace &&SMALL_DATA;

comment on table xpp_patches is
'Evidentary of release'
;
comment on column xpp_patches.seq_number    is 'Sequence number of release';
comment on column xpp_patches.patch_type    is 'Type of patch';
comment on column xpp_patches.patch_number  is 'Number of the patch';
comment on column xpp_patches.script_name   is 'Script name';
comment on column xpp_patches.created_by    is 'Author of the patch';
comment on column xpp_patches.create_date   is 'Date of ceration of the patch';
comment on column xpp_patches.applied_by    is 'Applied by Oracle user';
comment on column xpp_patches.apply_date    is 'Date of apply';
comment on column xpp_patches.description   is 'Description of the patch';


prompt create sequence seq_xpp_patches
create sequence seq_xpp_patches
  start with 1000
  maxvalue 9999999999999999999999999999
  minvalue 1
  nocycle
  cache 20
  noorder;
  
prompt create sequence seq_xpp_internal_msg
create sequence seq_xpp_internal_msg
  start with 1000
  maxvalue 9999999999999999999999999999
  minvalue 1
  nocycle
  cache 20
  noorder;


prompt create table xpp_conf_param
create table xpp_conf_param
(
   name         varchar2(1000 char)              not null
  ,val          varchar2(4000 char)
  ,description  varchar2(2000 char)
)
tablespace &&SMALL_DATA;


comment on table  xpp_conf_param             is 'Configuration of parameters';
comment on column xpp_conf_param.name        is 'Primary key - Name of configurable variable.';
comment on column xpp_conf_param.val         is 'Value of configurable variable.';
comment on column xpp_conf_param.description is 'Description of parameters';

prompt create sequence seq_xpp_conf_execution
create sequence seq_xpp_conf_execution
  start with 1000
  maxvalue 9999999999999999999999999999
  minvalue 1
  nocycle
  cache 20
  noorder;

prompt create table xpp_conf_execution
create table xpp_conf_execution
(
    id                     number        not null
   ,queue_name             varchar2(200) not null
   ,queue_name_short       varchar2(200) not null
   ,general_procedure_name varchar2(200) not null
   ,procedure_to_run       varchar2(200) not null
   ,procedure_on_success   varchar2(200)
   ,procedure_on_error     varchar2(200)
   ,param_on_error_input   number default 1 not null
   ,step                   number default 0 not null
   ,number_of_executors    number default 0 
   ,unique (queue_name, procedure_to_run, step)
   , primary key (id)
) tablespace &&MEDIUM_DATA;

comment on table  xpp_conf_execution                        IS 'List of execution procedures and steps';
comment on column xpp_conf_execution.id                     IS 'Primary ket generate by sequence seq_xpp_conf_execution';
comment on column xpp_conf_execution.queue_name             IS 'Name of queue';
comment on column xpp_conf_execution.queue_name_short       IS 'Name of queue short is only short name of queue without schema name and prefix';
comment on column xpp_conf_execution.general_procedure_name IS 'Name of general procedure to execute';
comment on column xpp_conf_execution.procedure_to_run       IS 'Name of procedure to execute';
comment on column xpp_conf_execution.procedure_on_success   IS 'Name of procedure to execute when execution of procedure_to_run is sucessfull';
comment on column xpp_conf_execution.procedure_on_error     IS 'Name of procedure to execute when execution of procedure_to_run is raising an error';
comment on column xpp_conf_execution.param_on_error_input   IS 'Number of parameter for procedure_on_error, when 1 only type is send, when 2 type and SQLCODE and SQLERROR, when 3 type and SQLCODE and SQLERROR and dbms_utility.format_error_backtrace is send';
comment on column xpp_conf_execution.number_of_executors    IS 'Number of requested parallel executed tasks';
comment on column xpp_conf_execution.step                   IS 'Step in queue';


prompt create table xpp_execution_chunk
create table xpp_execution_chunk
(
  task_name    varchar2(128)               not null,
  chunk_id     number                      not null,
  status       varchar2(100),
  start_rowid  rowid,
  end_rowid    rowid,
  start_id     number,
  end_id       number
) tablespace &&MEDIUM_DATA;

comment on table  xpp_execution_chunk             is 'Register of chunks of the tasks';
comment on column xpp_execution_chunk.task_name   is 'Name of the task';
comment on column xpp_execution_chunk.chunk_id    is 'Chunk identification';
comment on column xpp_execution_chunk.status      is 'Status of the chunk';
comment on column xpp_execution_chunk.start_rowid is 'Start rowid';
comment on column xpp_execution_chunk.end_rowid   is 'End rowid';
comment on column xpp_execution_chunk.start_id    is 'Start id';
comment on column xpp_execution_chunk.end_id      is 'End id';



--prompt Creation of XPP tables 
--@xpp_tables.sql

--prompt Creation of XPP sequences 
--@xpp_sequence.sql

prompt define request type.

create type xpp_type_request as object (  
   operation           varchar2(200)
  ,xpp_schema          varchar2(200)
  ,queue_name          varchar2(200)
  ,queue_table_name    varchar2(200)
  ,queue_type_name     varchar2(200)
  ,storage_clause      varchar2(500)
  ,queue_comments      varchar2(1000)
  ,user_to_grant       varchar2(200)
  ,name_of_procedure   varchar2(200)
  ,recipient_name      varchar2(200)
  ,seq_xpp_internal_msg varchar2(25)
)
/

prompt create queue table
begin
    dbms_aqadm.create_queue_table(
      queue_table        => 'xpp_table_request'
     ,queue_payload_type =>  'xpp_type_request'
     ,multiple_consumers => true
    );
end;
/

prompt create queue
begin
    dbms_aqadm.create_queue(
      queue_name  => 'xpp_queue_request'
     ,queue_table => 'xpp_table_request'
    );
end;
/

prompt create start queue
begin
    dbms_aqadm.start_queue(
      queue_name => 'xpp_queue_request'
    );
end;
/

--
-- we'll use that table to let our procedure 'communicate' with us -- our dequeue 
-- procedure that is called automagically 
--
-- prompt create procedure insert_xpp_request
-- -- a simple procedure to enqueue a message onto that queue
-- create or replace procedure insert_xpp_request(
--    p_operation        in varchar2
--   ,p_queue_name       in varchar2
--   ,p_queue_table_name in varchar2
--   ,p_queue_type_name  in varchar2
--   ,p_storage_clause   in varchar2
--   ,p_queue_comments   in varchar2
-- )
-- authid definer
-- is
--  enqueue_options dbms_aq.enqueue_options_t;
--  message_properties dbms_aq.message_properties_t;
--  message_handle raw(16);
--  message xppadm.xpp_type_request;
--
--  pragma autonomous_transaction;
-- begin
--    message := xpp_type_request(
--                          p_operation
--                         ,p_queue_name
--                         ,p_queue_table_name
--                         ,p_queue_type_name
--                         ,p_storage_clause
--                         ,p_queue_comments
--                      );
--    dbms_aq.enqueue(queue_name => 'xpp_queue_request',
--                    enqueue_options => enqueue_options,
--                    message_properties => message_properties,
--                    payload => message,
--                    msgid => message_handle);
--   commit;
-- end insert_xpp_request;
-- /


prompt create table xpp_log_trace
create table xpp_log_trace(
  id       number
 ,severity number
 ,procedure_name  varchar2(200)
 ,msg      varchar2(4000)
 ,log_time timestamp(6) default systimestamp
 ,log_date varchar2(8) default to_char(sysdate, 'yyyymmdd')
 ,primary key(id)
)
partition by hash (log_date)
(
  partition log_date
) 
tablespace &&MEDIUM_DATA;

comment on table xpp_log_trace is
'Log of processing for debug'
;
comment on column xpp_log_trace.id             is 'Identification to keep sequence of records in log';
comment on column xpp_log_trace.severity       is 'Severity level 1 - fatal, 2 - error, 3 - warning, 4 - info, 5 - debug';
comment on column xpp_log_trace.procedure_name is 'Name of procedure';
comment on column xpp_log_trace.msg            is 'Content of message';
comment on column xpp_log_trace.log_time       is 'Datetime of logged event';
comment on column xpp_log_trace.log_date       is 'Date of logged event. It is redundant information, but usefull while mantenece.';

prompt create sequence seq_xpp_log_trace_id
create sequence seq_xpp_log_trace_id
  start with 1000
  maxvalue 9999999999999999999999999999
  minvalue 1
  nocycle
  cache 20
  noorder;

set def off

prompt create xpp_log package
@@xpp_log.pks
@@xpp_log.pkb

prompt create xpp package
@@xpp_internal.pks
@@xpp_internal.pkb


prompt create procedure xpp_internal_process_request.proc
@@xpp_internal_process_request.proc

--prompt create procedure xpp_general_process_request
--@@xpp_general_process_request.proc


set def on

-- register recipient
begin
    dbms_aqadm.add_subscriber
    ( queue_name => '&&XPP_OWNER..xpp_queue_request',
      subscriber => sys.aq$_agent( 'recipient', null, null ) );
end;
/

-- -- ADDED: Create loopback database link
-- create database link aqtest.LoopBack connect to aqtest identified by aqtest
-- using 'funk92.us.oracle.com';
--
-- -- ADDED: Setup scheduling for messages
-- begin DBMS_AQADM.Schedule_Propagation(Queue_Name  => 'MSG_QUEUE',
--                                       Destination => 'aqtest.LoopBack',
--                                       Start_Time  => sysdate,
--                                       Latency     => 0);
-- end;
-- /

-- register callback procedure
begin
dbms_aq.register (
   sys.aq$_reg_info_list(
      sys.aq$_reg_info(
         '&&XPP_OWNER..xpp_queue_request:recipient',
         dbms_aq.namespace_aq,
         'plsql://&&XPP_OWNER..xpp_internal_process_request',
         hextoraw('ff')
         )
      ),
   1
   );
end;
/




---- and that should do it -- ready to go -- and we test:
--select * from out_result;
--
---- Expected 
---- no rows selected
--
--
---- insert message into queue
--begin
--  insert_xpp_request( 'This is a test.... ko' );
--end;
--/
--
--
--select * from out_result;
--
----Expected result:
----MSG
---- -------------------------------------------
----Dequeued and processed "This is a test...."




Set DEF off


prompt create xpp package
@@xpp.pks
@@xpp.pkb



Set DEF on

prompt grant xpp
grant execute on &&xpp_owner..xpp to &&xpp_user;

declare
  x number;
begin
  sys.dbms_job.submit
    (
      job        => x
     ,what       => 'begin xpp_log.clean_old_logs; end;'
     ,next_date  => trunc(sysdate)
     ,interval   => 'trunc(sysdate+7)'
     ,no_parse   => false
    );
--:jobnumber := to_char(x);
end;
/



prompt try to recompile all invalid objects
begin
  dbms_utility.compile_schema( '&&XPP_OWNER', false );
end;
/

prompt Insert initial configuration
insert into xpp_conf_param (name, val, description)
values ('LOG_LEVEL', '2',
'When value will be set to 5 will enable adding into log DEBUG, INFO, WARNING, ERROR, FATAL.
When value will be set to 4 will enable adding into log INFO, WARNING, ERROR, FATAL.
When value will be set to 3 will enable adding into log WARNING, ERROR, FATAL.
When value will be set to 2 will enable adding into log ERROR, FATAL.
When value will be set to 1 will enable adding into log FATAL.
');

insert into xpp_conf_param (name, val, description)
values ('MAX_DURATION', '150',
'Max time waiting time in sec before the request is raising exception like an timeout.');

insert into xpp_conf_param (name, val, description) values (
'CLEAN_OLDER_THEN_X_DAYS', '90', 
'The value is defined interval for maintaince job to clean old data in trace_log table');

Set DEF On

PROMPT *************************************************************************
BEGIN

-- FINAL UNIVERSAL SECTION ========================================================================

   DBMS_OUTPUT.PUT_LINE ('Log - Insert into XPP_PATCHES');
    INSERT INTO xpp_patches
          (
           seq_number,
           patch_type,
           patch_number,
           script_name,
           created_by,
           create_date,
           applied_by,
           apply_date,
           description
          )
    VALUES
          (
           seq_xpp_patches.NEXTVAL,
           DECODE('&patch_type','PR','AN','&patch_type'),
           TO_NUMBER('&patch_number'),
           '&script_name',
           '&created_by',
           TO_DATE('&create_date','DD.MM.YYYY'),
           USER,
           SYSDATE,
           '&description (Script version &version)'
          );

   DBMS_OUTPUT.PUT_LINE ('-------------------------');
   DBMS_OUTPUT.PUT_LINE ('End   ' || TO_CHAR(SYSDATE,'DD.MM.YYYY HH24:MI:SS') ) ;
   DBMS_OUTPUT.PUT_LINE ('-------------------------');

   DBMS_OUTPUT.PUT_LINE ('**********************************************');
   DBMS_OUTPUT.PUT_LINE ('Please check in the log file, whether all the');
   DBMS_OUTPUT.PUT_LINE ('database objects were created successfully.  ');
   DBMS_OUTPUT.PUT_LINE ('**********************************************');
   DBMS_OUTPUT.PUT_LINE ('* If yes, please C O M M I T all the changes *');
   DBMS_OUTPUT.PUT_LINE ('**********************************************');
   DBMS_OUTPUT.PUT_LINE ('* If not, please R O L L B A C K the changes *');
   DBMS_OUTPUT.PUT_LINE ('**********************************************');

EXCEPTION

   WHEN OTHERS THEN

      DBMS_OUTPUT.PUT_LINE(SQLERRM);
      ROLLBACK;
      DBMS_OUTPUT.PUT_LINE ('   ');
      DBMS_OUTPUT.PUT_LINE ('*******************************************');
      DBMS_OUTPUT.PUT_LINE ('Because of error ... R O L L E D   B A C K ');
      DBMS_OUTPUT.PUT_LINE ('*******************************************');

END;
/

SPOOL Off
Set SERVEROUTPUT Off
Set VERIFY On
Set TRIMSPOOL Off
UNDEF scrip_parameter

UNDEF XPP_OWNER
UNDEF XPP_USER

UNDEF MEDIUM_DATA
UNDEF TEMP_TABLESPACE_DATA
UNDEF MEDIUM_IDX

UNDEF SMALL_DATA
UNDEF SMALL_IDX

UNDEF XPP_USER_PASS_SUFFIX
UNDEF XPP_USER_PASS

UNDEF patch_type
UNDEF patch_number
UNDEF rest_of_name
UNDEF step_order
UNDEF description
UNDEF version
UNDEF created_by
UNDEF create_date
UNDEF affected_apps
