-- DESCRIBING SECTION =============================================================================
-------------------------------------------------------------------------------- 
-- 1. SOFTWAROVA s.r.o.
-- (c) 2012
--------------------------------------------------------------------------------
-- eXtreme PARALLEL PROCESSING
--------------------------------------------------------------------------------
-- Requirements:
-- Oracle Database Enterprise version >= 11g
-- CPU >= 1, Mem, Cluster
-- Sufficient size of REDO LOG is required.
--------------------------------------------------------------------------------
-- Run as user defined in XPP_USER
-- =============================================================================
------------- S C R I P T    N A M E -----------
DEF patch_type   = du
DEF patch_number = 00000002
DEF step_order   = 05
DEF rest_of_name = test_run_xppusr
------------- S C R I P T    I N F O -----------
DEF description     = 'Installation test script XPP'
DEF version         = 1.0
DEF created_by      = '1SW'
DEF create_date     = 07.06.2012
DEF affected_apps   = 'XPP'
REM for             = SQL*PLUS
REM parameters      = none

----------- S C R I P T    H I S T O R Y --------
REM     Version   When             Who               What
REM     1.0       2012/05/25       Petr Jezek        Creation
-- INITIAL UNIVERSAL SECTION ====================================================

@../du00000002_setting_env_xpp.sql

DEF script_name  = &patch_type&patch_number._&step_order&rest_of_name
Set SERVEROUTPUT On Size 1000000
SPOOL &script_name..log
Set VERIFY Off
Set TRIMSPOOL On
PROMPT
PROMPT *************************************************************************
PROMPT SCRIPT FILE: &script_name..sql
PROMPT LOG    FILE: &script_name..log
PROMPT SCRIPT DESC: &description
PROMPT SCRIPT VER.: &version
PROMPT *************************************************************************
PROMPT
PROMPT Affected Application(s) : &affected_apps
PROMPT Executing ...
PROMPT
-- EXECUTIVE SECTION ==============================================================================
Set FEEDBACK Off
Declare
  v_db_user VARCHAR2(500);
  v_net_user VARCHAR2(500);
Begin 
   -- Common actions
   SELECT 'Database user: ' || user || '@' || sys_context('USERENV', 'DB_NAME')
     INTO v_db_user 
     FROM dual;
   SELECT 'Network user: ' || sys_context('USERENV', 'OS_USER') || '@' || sys_context('USERENV', 'HOST')
     INTO v_net_user 
     FROM dual;
   DBMS_OUTPUT.put_line ('------------------------------------------------------------');
   DBMS_OUTPUT.put_line ('Start ' || TO_CHAR(SYSDATE,'DD.MM.YYYY HH24:MI:SS') ) ;
   DBMS_OUTPUT.put_line(v_db_user);
   DBMS_OUTPUT.put_line(v_net_user);   
   DBMS_OUTPUT.put_line ('------------------------------------------------------------');
   DBMS_OUTPUT.put_line ('	') ;
End; 
/
Set FEEDBACK On
Set DEF on
SET SERVEROUTPUT ON SIZE UNLIMITED

/* XPP example - Step 04: Event (open - process - wait - close) */
prompt Event (open - process - wait - close). Name of instance is 'test02'

prompt The processing will takes some time based on numbers of your CPU and configuration. (While calling XPP.event_open)
prompt The default example is using 3 parallel tasks.
prompt In the meantime you can browse sessions or using of CPU on your Oracle Enterprise Manager.
  
Set FEEDBACK On
Set DEF on
SET SERVEROUTPUT ON SIZE UNLIMITED

DECLARE 
    P_QUEUE_NAME        VARCHAR2(32767);
    P_PARALLEL_LEVEL    NUMBER;
    P_NAME_OF_PROC      VARCHAR2(32767);
    P_NAME_OF_PROC_OK   VARCHAR2(32767);
    P_NAME_OF_PROC_ERR  VARCHAR2(32767);

    P_MESSAGE         &&XPP_OWNER..type_test02;
    --
    l_message_id     number(5);
    l_subject        varchar2(100);
    l_text           varchar2(100);
    l_dollar_value   number(4,2);

  -- 
    --enqueue_options    sys.dbms_aq.enqueue_options_t;
    --message_properties sys.dbms_aq.message_properties_t;
    message_handle raw(16);
    
BEGIN 
  P_QUEUE_NAME := 'test02';
  P_PARALLEL_LEVEL := 3; -- number of required parallel executions
---------------------------------------------
  DBMS_OUTPUT.Put_Line('Test execute with full log of sucessful, failed, error_param with backtrace(p_param_on_error_input=3)');  
  
  P_NAME_OF_PROC     := '&&XPP_USER..TO_TEST';
  P_NAME_OF_PROC_OK  := '&&XPP_USER..on_success';
  P_NAME_OF_PROC_ERR := '&&XPP_USER..on_error3';

  &&XPP_OWNER..XPP.EVENT_OPEN ( 
     p_queue_name           => P_QUEUE_NAME 
    ,p_number_of_executors  => p_parallel_level 
    ,p_procedure_to_run     => P_NAME_OF_PROC
    ,p_procedure_on_success => P_NAME_OF_PROC_OK
    ,p_procedure_on_error   => P_NAME_OF_PROC_ERR
    ,p_param_on_error_input => 3
  );

  DBMS_OUTPUT.Put_Line('open sucessfull');
  COMMIT;

  for l_message_id in 1..1000
   loop
       l_subject        := 'Subject '||to_char(l_message_id);
       l_text           := 'XPP example '||to_char(l_message_id)||' ';
       --l_dollar_value   := l_message_id+10;
       
       P_MESSAGE        := &&XPP_OWNER..type_test02(
                              l_message_id
                             ,l_subject
                             ,l_text
                             ,l_dollar_value
                           );


     &&XPP_OWNER..xppi_test02.process( 
       p_queue_name => P_QUEUE_NAME
      ,p_message => P_MESSAGE
     );

     COMMIT;
   end loop;  
  DBMS_OUTPUT.Put_Line('end loop');

  &&XPP_OWNER..XPP.event_wait_processing_end( P_QUEUE_NAME);
  DBMS_OUTPUT.Put_Line('waiting finished sucessfull.');
  COMMIT;

  &&XPP_OWNER..XPP.event_close(P_QUEUE_NAME);
  DBMS_OUTPUT.Put_Line('close sucessfull');
  COMMIT;  
---------------------------------------------
  DBMS_OUTPUT.Put_Line('Test execute with full log of sucessful, failed, error_param with sqlcode and sqlerrm(p_param_on_error_input=2)');  

  P_NAME_OF_PROC_ERR := '&&XPP_USER..on_error2';

  &&XPP_OWNER..XPP.EVENT_OPEN ( 
     p_queue_name           => P_QUEUE_NAME 
    ,p_number_of_executors  => p_parallel_level 
    ,p_procedure_to_run     => P_NAME_OF_PROC
    ,p_procedure_on_success => P_NAME_OF_PROC_OK
    ,p_procedure_on_error   => P_NAME_OF_PROC_ERR
    ,p_param_on_error_input => 2
  );

  DBMS_OUTPUT.Put_Line('open sucessfull');
  COMMIT;


  for l_message_id in 1001..2000
   loop
       l_subject        := 'Subject '||to_char(l_message_id);
       l_text           := 'XPP example '||to_char(l_message_id)||' ';
       --l_dollar_value   := l_message_id+10;
       
       P_MESSAGE        := &&XPP_OWNER..type_test02(
                              l_message_id
                             ,l_subject
                             ,l_text
                             ,l_dollar_value
                           );


--     &&XPP_OWNER..xpp.event_process( 
--       p_queue_name => P_QUEUE_NAME
--      ,p_message => P_MESSAGE
--     );
     &&XPP_OWNER..xppi_test02.process( 
       p_queue_name => P_QUEUE_NAME
      ,p_message => P_MESSAGE
     );

     COMMIT;
   end loop;  
  DBMS_OUTPUT.Put_Line('end loop');
  
  &&XPP_OWNER..XPP.event_wait_processing_end( P_QUEUE_NAME);
  DBMS_OUTPUT.Put_Line('waiting finished sucessfull.');
  COMMIT;

  &&XPP_OWNER..XPP.event_close(P_QUEUE_NAME);
  DBMS_OUTPUT.Put_Line('close sucessfull');
  COMMIT;
  
---------------------------------------------
  DBMS_OUTPUT.Put_Line('Test execute with full log of sucessful, failed (p_param_on_error_input=1)');  

  P_NAME_OF_PROC_ERR := '&&XPP_USER..on_error1';

  &&XPP_OWNER..XPP.EVENT_OPEN ( 
     p_queue_name           => P_QUEUE_NAME 
    ,p_number_of_executors  => p_parallel_level 
    ,p_procedure_to_run     => P_NAME_OF_PROC
    ,p_procedure_on_success => P_NAME_OF_PROC_OK
    ,p_procedure_on_error   => P_NAME_OF_PROC_ERR
    ,p_param_on_error_input => 1
  );

  DBMS_OUTPUT.Put_Line('open sucessfull');
  COMMIT;


  for l_message_id in 2001..3000
   loop
       l_subject        := 'Subject '||to_char(l_message_id);
       l_text           := 'XPP example '||to_char(l_message_id)||' ';
       --l_dollar_value   := l_message_id+10;
       
       P_MESSAGE        := &&XPP_OWNER..type_test02(
                              l_message_id
                             ,l_subject
                             ,l_text
                             ,l_dollar_value
                           );


--     &&XPP_OWNER..xpp.event_process( 
--       p_queue_name => P_QUEUE_NAME
--      ,p_message => P_MESSAGE
--     );
     &&XPP_OWNER..xppi_test02.process( 
       p_queue_name => P_QUEUE_NAME
      ,p_message => P_MESSAGE
     );

     COMMIT;
   end loop;  
  DBMS_OUTPUT.Put_Line('end loop');
  
  &&XPP_OWNER..XPP.event_wait_processing_end( P_QUEUE_NAME);
  DBMS_OUTPUT.Put_Line('waiting finished sucessfull.');
  COMMIT;

  &&XPP_OWNER..XPP.event_close(P_QUEUE_NAME);
  DBMS_OUTPUT.Put_Line('close sucessfull');
  COMMIT;
  
---------------------------------------------
  DBMS_OUTPUT.Put_Line('Test execute with full log of sucessful');
  
  &&XPP_OWNER..XPP.EVENT_OPEN ( 
     p_queue_name           => P_QUEUE_NAME 
    ,p_number_of_executors  => p_parallel_level 
    ,p_procedure_to_run     => P_NAME_OF_PROC
    ,p_procedure_on_success => P_NAME_OF_PROC_OK
  );

  DBMS_OUTPUT.Put_Line('open sucessfull');
  COMMIT;


  for l_message_id in 3001..4000
   loop
       l_subject        := 'Subject '||to_char(l_message_id);
       l_text           := 'XPP example '||to_char(l_message_id)||' ';
       --l_dollar_value   := l_message_id+10;
       
       P_MESSAGE        := &&XPP_OWNER..type_test02(
                              l_message_id
                             ,l_subject
                             ,l_text
                             ,l_dollar_value
                           );


--     &&XPP_OWNER..xpp.event_process( 
--       p_queue_name => P_QUEUE_NAME
--      ,p_message => P_MESSAGE
--     );
     &&XPP_OWNER..xppi_test02.process( 
       p_queue_name => P_QUEUE_NAME
      ,p_message => P_MESSAGE
     );

     COMMIT;
   end loop;  
  DBMS_OUTPUT.Put_Line('end loop');
  
  &&XPP_OWNER..XPP.event_wait_processing_end( P_QUEUE_NAME);
  DBMS_OUTPUT.Put_Line('waiting finished sucessfull.');
  COMMIT;

  &&XPP_OWNER..XPP.event_close(P_QUEUE_NAME);
  DBMS_OUTPUT.Put_Line('close sucessfull');
  COMMIT;
  
---------------------------------------------
  DBMS_OUTPUT.Put_Line('Test only to execute parallel.');

  &&XPP_OWNER..XPP.EVENT_OPEN ( 
     p_queue_name           => P_QUEUE_NAME 
    ,p_number_of_executors  => p_parallel_level 
    ,p_procedure_to_run     => P_NAME_OF_PROC
  );

  DBMS_OUTPUT.Put_Line('open sucessfull');
  COMMIT;


  for l_message_id in 4001..5000
   loop
       l_subject        := 'Subject '||to_char(l_message_id);
       l_text           := 'XPP example '||to_char(l_message_id)||' ';
       --l_dollar_value   := l_message_id+10;
       
       P_MESSAGE        := &&XPP_OWNER..type_test02(
                              l_message_id
                             ,l_subject
                             ,l_text
                             ,l_dollar_value
                           );


--     &&XPP_OWNER..xpp.event_process( 
--       p_queue_name => P_QUEUE_NAME
--      ,p_message => P_MESSAGE
--     );
     &&XPP_OWNER..xppi_test02.process( 
       p_queue_name => P_QUEUE_NAME
      ,p_message => P_MESSAGE
     );

     COMMIT;
   end loop;  
  DBMS_OUTPUT.Put_Line('end loop');
  
  &&XPP_OWNER..XPP.event_wait_processing_end( P_QUEUE_NAME);
  DBMS_OUTPUT.Put_Line('waiting finished sucessfull.');
  COMMIT;

  &&XPP_OWNER..XPP.event_close(P_QUEUE_NAME);
  DBMS_OUTPUT.Put_Line('close sucessfull');
  COMMIT;

END; 
/



Set DEF On

PROMPT *************************************************************************
BEGIN

-- FINAL UNIVERSAL SECTION ========================================================================


   DBMS_OUTPUT.PUT_LINE ('-------------------------');
   DBMS_OUTPUT.PUT_LINE ('End   ' || TO_CHAR(SYSDATE,'DD.MM.YYYY HH24:MI:SS') ) ;
   DBMS_OUTPUT.PUT_LINE ('-------------------------');

   DBMS_OUTPUT.PUT_LINE ('**********************************************');
   DBMS_OUTPUT.PUT_LINE ('Please check in the log file, whether all the');
   DBMS_OUTPUT.PUT_LINE ('database objects were created successfully.  ');
   DBMS_OUTPUT.PUT_LINE ('**********************************************');
   DBMS_OUTPUT.PUT_LINE ('* If yes, please C O M M I T all the changes *');
   DBMS_OUTPUT.PUT_LINE ('**********************************************');
   DBMS_OUTPUT.PUT_LINE ('* If not, please R O L L B A C K the changes *');
   DBMS_OUTPUT.PUT_LINE ('**********************************************');

EXCEPTION

   WHEN OTHERS THEN

      DBMS_OUTPUT.PUT_LINE(SQLERRM);
      ROLLBACK;
      DBMS_OUTPUT.PUT_LINE ('   ');
      DBMS_OUTPUT.PUT_LINE ('*******************************************');
      DBMS_OUTPUT.PUT_LINE ('Because of error ... R O L L E D   B A C K ');
      DBMS_OUTPUT.PUT_LINE ('*******************************************');

END;
/

SPOOL Off
Set SERVEROUTPUT Off
Set VERIFY On
Set TRIMSPOOL Off
UNDEF scrip_parameter

UNDEF XPP_OWNER
UNDEF XPP_USER

UNDEF MEDIUM_DATA
UNDEF TEMP_TABLESPACE_DATA
UNDEF MEDIUM_IDX

UNDEF SMALL_DATA
UNDEF SMALL_IDX

UNDEF patch_type
UNDEF patch_number
UNDEF rest_of_name
UNDEF step_order
UNDEF description
UNDEF version
UNDEF created_by
UNDEF create_date
UNDEF affected_apps
