-- DESCRIBING SECTION =============================================================================
-------------------------------------------------------------------------------- 
-- 1. SOFTWAROVA s.r.o.
-- (c) 2012
--------------------------------------------------------------------------------
-- eXtreme Parallel Processing
--------------------------------------------------------------------------------
-- Requirements:
-- Oracle Database Enterprise version >= 11g
-- CPU >= 1, Mem, Cluster
-- Sufficient size of REDO LOG is required.
--------------------------------------------------------------------------------
-- Run as user defined in XPP_OWNER
-- =============================================================================
------------- S C R I P T    N A M E -----------
DEF patch_type   = du
DEF patch_number = 00000002
DEF step_order   = 01
DEF rest_of_name = _uninstall_xpp
------------- S C R I P T    I N F O -----------
DEF description     = 'Uninstallation script XPP'
DEF version         = 1.0
DEF created_by      = '1SW'
DEF create_date     = 07.06.2012
DEF affected_apps   = 'XPP'
REM for             = SQL*PLUS
REM parameters      = none

----------- S C R I P T    H I S T O R Y --------
REM     Version   When             Who               What
REM     1.0       2012/05/25       Petr Jezek        Creation
-- INITIAL UNIVERSAL SECTION ====================================================

@../du00000002_setting_env_xpp.sql

DEF script_name  = &patch_type&patch_number._&step_order&rest_of_name
Set SERVEROUTPUT On Size 1000000
SPOOL &script_name..log
Set VERIFY Off
Set TRIMSPOOL On
PROMPT
PROMPT *************************************************************************
PROMPT SCRIPT FILE: &script_name..sql
PROMPT LOG    FILE: &script_name..log
PROMPT SCRIPT DESC: &description
PROMPT SCRIPT VER.: &version
PROMPT *************************************************************************
PROMPT
PROMPT Affected Application(s) : &affected_apps
PROMPT Executing ...
PROMPT
-- EXECUTIVE SECTION ==============================================================================
Set FEEDBACK Off
Declare
  v_db_user VARCHAR2(500);
  v_net_user VARCHAR2(500);
Begin 
   -- Common actions
   SELECT 'Database user: ' || user || '@' || sys_context('USERENV', 'DB_NAME')
     INTO v_db_user 
     FROM dual;
   SELECT 'Network user: ' || sys_context('USERENV', 'OS_USER') || '@' || sys_context('USERENV', 'HOST')
     INTO v_net_user 
     FROM dual;
   DBMS_OUTPUT.put_line ('------------------------------------------------------------');
   DBMS_OUTPUT.put_line ('Start ' || TO_CHAR(SYSDATE,'DD.MM.YYYY HH24:MI:SS') ) ;
   DBMS_OUTPUT.put_line(v_db_user);
   DBMS_OUTPUT.put_line(v_net_user);   
   DBMS_OUTPUT.put_line ('------------------------------------------------------------');
   DBMS_OUTPUT.put_line ('	') ;
End; 
/
Set FEEDBACK On
Set DEF on
SET SERVEROUTPUT ON SIZE UNLIMITED

prompt create stop queue
begin
    dbms_aqadm.stop_queue(
      queue_name => '&&XPP_OWNER..xpp_queue_request'
    );
end;
/

prompt drop queue
begin
    dbms_aqadm.drop_queue(
      queue_name  => '&&XPP_OWNER..xpp_queue_request'
    );
end;
/

prompt drop queue table
begin
    dbms_aqadm.drop_queue_table(
      queue_table        => '&&XPP_OWNER..xpp_table_request'
    );
end;
/

prompt Remove job for cleaning logs
declare
  l_job_id number;
begin
  select job
    into l_job_id
  from sys.user_jobs
  where 1=1
    and schema_user = '&&XPP_OWNER.' 
    and what = 'begin xpp_log.clean_old_logs; end;'
  ;
  
  sys.dbms_job.remove
    (
      job        => l_job_id
    );
end;
/ 

-- prompt unregister callback procedure
-- begin
-- dbms_aq.unregister (
--    sys.aq$_reg_info_list(
--       sys.aq$_reg_info(
--          '&&XPP_OWNER..xpp_queue_request:recipient',
--          dbms_aq.namespace_aq,
--          'plsql://&&XPP_OWNER..xpp_internal_process_request',
--          hextoraw('ff')
--          )
--       ),
--    1
--    );
-- end;
-- /
-- 
-- prompt unregister recipient
-- begin
--     dbms_aqadm.remove_subscriber
--     ( queue_name => '&&XPP_OWNER..xpp_queue_request',
--       subscriber => sys.aq$_agent( 'recipient', null, null ) );
-- end;
-- /

prompt drop package xpp
drop package xpp;


prompt drop procedure xpp_internal_process_request
drop procedure xpp_internal_process_request;

prompt drop package xpp_internal
drop package xpp_internal;

prompt drop package xpp_log
drop package xpp_log;

prompt drop type xpp_type_request
drop type xpp_type_request;


prompt drop sequence
drop sequence seq_xpp_conf_execution;
drop sequence seq_xpp_internal_msg;
drop sequence seq_xpp_log_trace_id;
drop sequence seq_xpp_patches;


prompt drop tables
drop table xpp_execution_chunk;
drop table xpp_conf_execution;
drop table xpp_conf_param;
drop table xpp_log_trace;
drop table xpp_patches;



Set DEF On

PROMPT *************************************************************************
BEGIN

-- FINAL UNIVERSAL SECTION ========================================================================

   DBMS_OUTPUT.PUT_LINE ('Log - Insert into XPP_PATCHES');
--     INSERT INTO xpp_patches
--           (
--            seq_number,
--            patch_type,
--            patch_number,
--            script_name,
--            created_by,
--            create_date,
--            applied_by,
--            apply_date,
--            description
--           )
--     VALUES
--           (
--            seq_xpp_patches.NEXTVAL,
--            DECODE('&patch_type','PR','AN','&patch_type'),
--            TO_NUMBER('&patch_number'),
--            '&script_name',
--            '&created_by',
--            TO_DATE('&create_date','DD.MM.YYYY'),
--            USER,
--            SYSDATE,
--            '&description (Script version &version)'
--           );

   DBMS_OUTPUT.PUT_LINE ('-------------------------');
   DBMS_OUTPUT.PUT_LINE ('End   ' || TO_CHAR(SYSDATE,'DD.MM.YYYY HH24:MI:SS') ) ;
   DBMS_OUTPUT.PUT_LINE ('-------------------------');

   DBMS_OUTPUT.PUT_LINE ('**********************************************');
   DBMS_OUTPUT.PUT_LINE ('Please check in the log file, whether all the');
   DBMS_OUTPUT.PUT_LINE ('database objects were created successfully.  ');
   DBMS_OUTPUT.PUT_LINE ('**********************************************');
   DBMS_OUTPUT.PUT_LINE ('* If yes, please C O M M I T all the changes *');
   DBMS_OUTPUT.PUT_LINE ('**********************************************');
   DBMS_OUTPUT.PUT_LINE ('* If not, please R O L L B A C K the changes *');
   DBMS_OUTPUT.PUT_LINE ('**********************************************');

EXCEPTION

   WHEN OTHERS THEN

      DBMS_OUTPUT.PUT_LINE(SQLERRM);
      ROLLBACK;
      DBMS_OUTPUT.PUT_LINE ('   ');
      DBMS_OUTPUT.PUT_LINE ('*******************************************');
      DBMS_OUTPUT.PUT_LINE ('Because of error ... R O L L E D   B A C K ');
      DBMS_OUTPUT.PUT_LINE ('*******************************************');

END;
/

SPOOL Off
Set SERVEROUTPUT Off
Set VERIFY On
Set TRIMSPOOL Off
UNDEF scrip_parameter

UNDEF XPP_OWNER
UNDEF XPP_USER

UNDEF MEDIUM_DATA
UNDEF TEMP_TABLESPACE_DATA
UNDEF MEDIUM_IDX

UNDEF SMALL_DATA
UNDEF SMALL_IDX

UNDEF XPP_USER_PASS_SUFFIX
UNDEF XPP_USER_PASS

UNDEF patch_type
UNDEF patch_number
UNDEF rest_of_name
UNDEF step_order
UNDEF description
UNDEF version
UNDEF created_by
UNDEF create_date
UNDEF affected_apps
